@@ -16,13 +16,14 @@ object can also navigate 'forward' and 'back', similar to a browser.
 All results returned from PMP endpoints are returned as :class:`NavigableDoc`
 objects, so the API for :class:`NavigableDoc` is important to look at as well.
 """
-
 import requests
 
 from .core.auth import PmpAuth
 from .core.conn import PmpConnector
+from .core.exceptions import BadQuery
 from .core.exceptions import NoToken
 from .collectiondoc.navigabledoc import NavigableDoc
+from .utils.json_utils import filter_dict
 
 
 class Client(object):
@@ -61,30 +62,31 @@ class Client(object):
         self.pager = None
         self.document = None
 
-    def gain_access(self, client_id, client_secret):
+    def gain_access(self, client_id,
+                    client_secret,
+                    auth_urn="urn:collectiondoc:form:issuetoken"):
         """Requests access for `entry_point` using provided authentication.
-        Finds the urn `urn:collectiondoc:form:issuetoken` and requests a
-        token using the protocol listed there.
+        Finds the `auth_urn` and requests a token using the protocol listed
+        there.
         """
-        AUTH_URN = "urn:collectiondoc:form:issuetoken"
         resp = requests.get(self.entry_point)
         home_doc = resp.json()
         self.document = NavigableDoc(home_doc)
-        auth_schema = self.document.options(AUTH_URN)
+        auth_schema = self.document.options(auth_urn)
         access_token_url = auth_schema.get('href', None)
         if not access_token_url:
             errmsg = "Missing authentication URL at endpoint."
             errmsg += " Review API values at {0} with options {1}"
-            raise NoToken(errmsg.format(AUTH_URN, str(auth_schema)))
+            raise NoToken(errmsg.format(auth_urn, str(auth_schema)))
 
         authorizer = PmpAuth(client_id, client_secret)
         try:
             authorizer.get_access_token2(access_token_url)
             self.connector = PmpConnector(authorizer)
-        except NoToken:
+        except NoToken as exc:
             errmsg = "Client connection failed. Check entry_point or"
             errmsg += " authentication schema used."
-            raise NoToken(errmsg)
+            raise NoToken(errmsg) from exc
 
     def get(self, endpoint):
         """Returns NavigableDoc object obtained from requested endpoint.
@@ -117,16 +119,59 @@ class Client(object):
             self.pager = self.document.pager
             return self.document
 
+    def save(self, endpoint, document):
+        """Saves a document (a string value) at PMP.
+        Args:
+
+           `endpoint` -- URL endpoint for saving documents
+           `document` -- data (str) to send over as a document payload.
+        """
+        results = self.connector.put(endpoint, document)
+        return results
+
+    def delete(self, document):
+        """Deletes a document from PMP API: simply fires 'DELETE'
+        to provided document's href endpoint. If permissions allow it, it
+        should delete and return True.
+
+        Args:
+           `document` -- NavigableDoc document to be deleted from PMP
+        """
+        return self.connector.delete(document.collectiondoc.get('href'))
+
+    # def upload(self, endpoint, upload_document):
+    #     """Uploads a rich media object to PMP API.
+    #     -- Not implemented yet.
+    #     """
+    #     raise Exception("Not implemented yet.")
+
     def query(self, rel_type, params=None):
         """Issues request for a query using urn with params to create
         a well-formed request.
+
+        Args:
+           `rel_type` -- Relation Type (urn)
+
+        Kwargs:
+           `params` -- Dictionary of params to construct a query
         """
-        return self.get(self.document.query(rel_type, params=params))
+        pmp_request = self.document.query(rel_type, params=params)
+        if pmp_request is None:
+            errmsg = "Can't create request for {0} with params {1}. Check that"
+            errmsg += " {0} is present in docuemnt."
+            raise BadQuery(errmsg.format(rel_type, str(params)))
+        else:
+            return self.get(pmp_request)
 
     def home(self):
         """Requests API home-doc `entry_point` and returns results.
         """
-        return self.get(self.entry_point)
+        if self.connector is not None and self.connector.authorized:
+            return self.get(self.entry_point)
+        else:
+            # Fragile: fix or reject and make all requests be authenticated
+            self.document = NavigableDoc(requests.get(self.entry_point).json())
+            return self.document
 
     def next(self):
         """Requests the `next` page listed by navigation. If
